@@ -2,7 +2,7 @@
  *
  *  This file handles the GUI. It "owns" the model and cfg.
  *
- *  Copyright (c) 2007 Diego Ongaro <ongardie@gmail.com>
+ *  Copyright (c) 2007-2008 Diego Ongaro <ongardie@gmail.com>
  *
  *  Largely based on:
  *
@@ -18,6 +18,12 @@
  *     launcher.c - (xfce4-panel plugin that opens programs)
  *     Copyright (c) 2005-2007 Jasper Huijsmans <jasper@xfce.org>
  *     Copyright (c) 2006-2007 Nick Schermer <nick@xfce.org>
+ * 
+ *  Popup command code adapted from:
+ *   - windowlist plugin
+ *     windowlist.c - (xfce4-panel plugin that lists open windows)
+ *     Copyright (c) 2002-2006  Olivier Fourdan
+ *     Copyright (c) 2007       Mike Massonnet <mmassonnet@xfce.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,12 +46,17 @@
 
 #include <gtk/gtk.h>
 
+#define USE_GTK_TOOLTIP_API     GTK_CHECK_VERSION(2,12,0)
+
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4panel/xfce-panel-plugin.h>
 #include <libxfce4panel/xfce-panel-convenience.h>
 #include <libxfce4panel/xfce-hvbox.h>
 #include <libxfcegui4/libxfcegui4.h>
 
+#define EXO_API_SUBJECT_TO_CHANGE
+#include <exo/exo.h>
+
 #include <string.h>
 
 #include "view.h"
@@ -55,6 +66,17 @@
 #include "model_system.h"
 #include "model_volumes.h"
 #include "model_user.h"
+#include "xfce4-popup-places.h"
+#include "button.h"
+
+struct _PlacesViewCfgIface
+{
+    PlacesView          *places_view;
+
+    void                (*update_menu)              (PlacesView*);
+    void                (*update_button)            (PlacesView*);
+    void                (*reconfigure_model)        (PlacesView*);
+};
 
 struct _PlacesView
 {
@@ -63,25 +85,23 @@ struct _PlacesView
     
     /* configuration */
     PlacesCfg                 *cfg;
-    PlacesCfgViewIface        *cfg_iface;
     PlacesViewCfgIface        *view_cfg_iface;
     
     /* view */
     GtkWidget                 *button;
-    GtkWidget                 *button_image;
-    GtkWidget                 *button_label;
     GtkWidget                 *menu;
-    GtkTooltips               *tooltips;
-    gboolean                  needs_separator;
-    guint                     menu_timeout_id;
-    
-    GtkOrientation            orientation;
-    gint                      size;
-    gboolean                  show_button_icon;
-    gboolean                  show_button_label;
-    gchar                     *label_tooltip_text;
-    gboolean                  force_update_theme;
 
+#if !USE_GTK_TOOLTIP_API
+    GtkTooltips               *tooltips;
+#endif
+
+#if USE_RECENT_DOCUMENTS
+    gulong                     recent_manager_changed_handler;
+#endif
+
+    gboolean                   needs_separator;
+    guint                      menu_timeout_id;
+    
     /* model */
     GList                     *bookmark_groups;
 };
@@ -139,12 +159,6 @@ places_view_cfg_iface_reconfigure_model(PlacesViewCfgIface *iface)
     iface->reconfigure_model(iface->places_view);
 }
 
-inline GtkWidget*
-places_view_cfg_iface_make_empty_cfg_dialog(PlacesViewCfgIface *iface)
-{
-    return iface->make_empty_cfg_dialog(iface->places_view);
-}
-
 /********** Model Management **********/
 static void
 pview_destroy_model(PlacesView *view)
@@ -161,7 +175,7 @@ pview_destroy_model(PlacesView *view)
         do{
             if(bookmark_group_li->data != NULL){
                 bookmark_group = (PlacesBookmarkGroup*) bookmark_group_li->data;
-                places_bookmark_group_finalize(bookmark_group);
+                places_bookmark_group_destroy(bookmark_group);
             }
 
             bookmark_group_li = bookmark_group_li->next;
@@ -212,40 +226,31 @@ pview_model_changed(GList *bookmark_groups)
 }
 
 
+static void
+pview_bookmark_action_call_wrapper(PlacesView *view, PlacesBookmarkAction *action)
+{
+    g_assert(action != NULL);
+
+    if(action->may_block){
+
+        gtk_widget_set_sensitive(view->button, FALSE);
+
+        while(gtk_events_pending())
+            gtk_main_iteration();
+
+        places_bookmark_action_call(action);
+    
+        gtk_widget_set_sensitive(view->button, TRUE);
+
+    }else{
+        places_bookmark_action_call(action);
+    }
+}
+
 /********** Gtk Callbacks **********/
 
 /* Panel callbacks */
 
-static gboolean
-pview_cb_size_changed(PlacesView *pd)
-{
-    g_assert(pd != NULL);
-    g_assert(pd->button != NULL);
-
-    if(GTK_WIDGET_REALIZED(pd->button))
-        pview_button_update(pd);
-
-    return TRUE;
-}
-
-static void
-pview_cb_theme_changed(PlacesView *view)
-{
-    g_assert(view != NULL);
-    g_assert(view->button != NULL);
-
-    DBG("theme changed");
-
-    /* update the button */
-    if(GTK_WIDGET_REALIZED(view->button)){
-        view->force_update_theme = TRUE;
-        pview_button_update(view);
-    }
-    
-    /* force a menu update */
-    pview_destroy_menu(view);
-}
-
 /* Menu callbacks */
 static gboolean /* return false to stop calling it */
 pview_cb_menu_timeout(PlacesView *pd){
@@ -378,13 +383,13 @@ pview_cb_menu_item_context_act(GtkWidget *item, PlacesView *pd)
 
     /* we want the menu gone - now - since it prevents mouse grabs */
     gtk_menu_shell_deactivate(GTK_MENU_SHELL(pd->menu));
-    while(g_main_context_iteration(NULL, FALSE))
-        /* no op */;
+    while(gtk_events_pending())
+        gtk_main_iteration();
 
     action = (PlacesBookmarkAction*) g_object_get_data(G_OBJECT(item), "action");
     DBG("Calling action %s", action->label);
-    places_bookmark_action_call(action);
-    
+    pview_bookmark_action_call_wrapper(pd, action);
+
 }
 
 static gboolean
@@ -443,6 +448,14 @@ pview_cb_menu_item_press(GtkWidget *menu_item, GdkEventButton *event, PlacesView
         return FALSE;
 }
 
+static void
+pview_cb_menu_item_activate(GtkWidget *menu_item, PlacesView *view)
+{
+    PlacesBookmark *bookmark = (PlacesBookmark*) g_object_get_data(G_OBJECT(menu_item), "bookmark");
+
+    pview_bookmark_action_call_wrapper(view, bookmark->primary_action);
+}
+
 /* Recent Documents */
 
 #if USE_RECENT_DOCUMENTS
@@ -454,14 +467,50 @@ pview_cb_recent_item_open(GtkRecentChooser *chooser, PlacesView *pd)
     g_free(uri);
 }
 
+static void
+pview_cb_recent_changed(GtkRecentManager *recent_manager, GtkWidget *recent_menu)
+{
+    GtkWidget *recent_item;
+    int recent_count = 0;
+    
+    g_object_get(recent_manager,
+                 "size", &recent_count,
+                 NULL);
+
+    recent_item = gtk_menu_get_attach_widget(GTK_MENU(recent_menu));
+    if (GTK_IS_WIDGET(recent_item))
+        gtk_widget_set_sensitive(recent_item, recent_count > 0);
+
+    if (recent_count == 0) {
+        gtk_menu_popdown(GTK_MENU(recent_menu));
+    }
+    else {
+        while (gtk_events_pending())
+            gtk_main_iteration();
+
+        gtk_menu_reposition(GTK_MENU(recent_menu));
+    }
+}
+
 static gboolean
-pview_cb_recent_items_clear(GtkWidget *clear_item)
+pview_cb_recent_items_clear(GtkWidget *clear_item, GtkWidget *recent_menu)
 {
     GtkRecentManager *manager = gtk_recent_manager_get_default();
+    
     gint removed = gtk_recent_manager_purge_items(manager, NULL);
     DBG("Cleared %d recent items", removed);
+
+    pview_cb_recent_changed(manager, recent_menu);
+    
     return TRUE;
 }
+
+static gboolean
+pview_cb_recent_items_clear3(GtkWidget *clear_item, GdkEventButton *event, GtkWidget *recent_menu)
+{
+    return pview_cb_recent_items_clear(clear_item, recent_menu);
+}
+
 #endif
 
 
@@ -470,8 +519,21 @@ pview_cb_recent_items_clear(GtkWidget *clear_item)
 static void
 pview_destroy_menu(PlacesView *view)
 {
-    if(view->menu != NULL){
+#if USE_RECENT_DOCUMENTS
+    GtkRecentManager *recent_manager = gtk_recent_manager_get_default();
+#endif
+
+    if(view->menu != NULL) {
         gtk_menu_shell_deactivate(GTK_MENU_SHELL(view->menu));
+
+#if USE_RECENT_DOCUMENTS
+        if (view->recent_manager_changed_handler) {
+            g_signal_handler_disconnect(recent_manager,
+                                        view->recent_manager_changed_handler);
+            view->recent_manager_changed_handler = 0;
+        }
+#endif
+
         gtk_widget_destroy(view->menu);
         view->menu = NULL;
     }
@@ -520,9 +582,9 @@ pview_add_menu_item(PlacesView *view, PlacesBookmark *bookmark)
 
     if(bookmark->primary_action != NULL){
 
-        g_signal_connect_swapped(item, "activate",
-                                 G_CALLBACK(places_bookmark_action_call), 
-                                 bookmark->primary_action);
+        g_signal_connect(item, "activate",
+                         G_CALLBACK(pview_cb_menu_item_activate),
+                         view);
 
     }
     if(bookmark->force_gray || bookmark->primary_action == NULL){
@@ -534,7 +596,7 @@ pview_add_menu_item(PlacesView *view, PlacesBookmark *bookmark)
     }
 
     g_signal_connect_swapped(item, "destroy",
-                     G_CALLBACK(places_bookmark_free), bookmark);
+                     G_CALLBACK(places_bookmark_destroy), bookmark);
 
     gtk_menu_shell_append(GTK_MENU_SHELL(view->menu), item);
     gtk_widget_show(item);
@@ -554,6 +616,7 @@ pview_update_menu(PlacesView *pd)
     GtkWidget *recent_menu;
     GtkWidget *clear_item;
     GtkWidget *recent_item;
+    GtkRecentManager *recent_manager = gtk_recent_manager_get_default();
 #endif
 
     DBG("destroy menu");
@@ -567,7 +630,8 @@ pview_update_menu(PlacesView *pd)
     pd->menu = gtk_menu_new();
     
     /* make sure the menu popups up in right screen */
-    gtk_menu_attach_to_widget(GTK_MENU(pd->menu), pd->button, NULL);
+    /* need exo_noop for GTK 2.6 and 2.8; starting with 2.10, NULL is OK */
+    gtk_menu_attach_to_widget(GTK_MENU(pd->menu), pd->button, (GtkMenuDetachFunc) exo_noop);
     gtk_menu_set_screen(GTK_MENU(pd->menu),
                         gtk_widget_get_screen(pd->button));
 
@@ -637,7 +701,7 @@ pview_update_menu(PlacesView *pd)
 
         g_signal_connect(recent_menu, "item-activated", 
                          G_CALLBACK(pview_cb_recent_item_open), pd);
-    
+            
         if(pd->cfg->show_recent_clear){
 
             separator = gtk_separator_menu_item_new();
@@ -657,10 +721,10 @@ pview_update_menu(PlacesView *pd)
 
             /* try button-release-event to catch mouse clicks and not hide the menu after */
             g_signal_connect(clear_item, "button-release-event",
-                             G_CALLBACK(pview_cb_recent_items_clear), NULL);
+                             G_CALLBACK(pview_cb_recent_items_clear3), recent_menu);
             /* use activate when button-release-event doesn't catch it (e.g., enter key pressed) */
             g_signal_connect(clear_item, "activate",
-                             G_CALLBACK(pview_cb_recent_items_clear), NULL);
+                             G_CALLBACK(pview_cb_recent_items_clear), recent_menu);
     
         }
     
@@ -669,12 +733,16 @@ pview_update_menu(PlacesView *pd)
             gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(recent_item), 
                                           gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU));
         }
-
+        
         gtk_menu_item_set_submenu(GTK_MENU_ITEM(recent_item), recent_menu);
         gtk_widget_show(recent_menu);
         
         gtk_menu_shell_append(GTK_MENU_SHELL(pd->menu), recent_item);
         gtk_widget_show(recent_item);
+
+        pd->recent_manager_changed_handler = g_signal_connect(recent_manager, "changed",
+                                                              G_CALLBACK(pview_cb_recent_changed), recent_menu);
+        pview_cb_recent_changed(recent_manager, recent_menu);
     }
 #endif
 
@@ -727,208 +795,94 @@ pview_open_menu(PlacesView *pd)
     }
 }
 
+static GdkPixbuf*
+pview_pixbuf_factory(gint size)
+{
+    return xfce_themed_icon_load_category(2, size);
+}
+
 static void
 pview_button_update(PlacesView *view)
 {
-    
     PlacesCfg *cfg = view->cfg;
-    gboolean orientation_changed, size_changed, 
-             icon_presence_changed, label_presence_changed, 
-             label_tooltip_changed, theme_changed;
-    static gboolean first_run = TRUE;
+    PlacesButton *button = (PlacesButton*) view->button;
 
-    DBG("button_update (first run: %1x)", first_run);
+    static guint tooltip_text_hash = 0;
+    guint new_tooltip_text_hash;
 
-    if(first_run){
-        first_run = FALSE;
-        
-        orientation_changed = TRUE;
-        view->orientation = xfce_panel_plugin_get_orientation(view->plugin);
-        
-        size_changed = TRUE;
-        view->size = xfce_panel_plugin_get_size(view->plugin);
-        
-        icon_presence_changed = TRUE;
-        view->show_button_icon = cfg->show_button_icon;
-        
-        label_presence_changed = TRUE;
-        view->show_button_label = cfg->show_button_label;
-        
-        label_tooltip_changed = TRUE;
-        view->label_tooltip_text = g_strdup(cfg->label);
+    places_button_set_label(button,
+                  cfg->show_button_label ? cfg->label : NULL);
 
-        theme_changed = TRUE;
-        view->force_update_theme = FALSE;
+    places_button_set_pixbuf_factory(button,
+                  cfg->show_button_icon  ? pview_pixbuf_factory : NULL);
 
-    }else{
-        orientation_changed    = (view->orientation != xfce_panel_plugin_get_orientation(view->plugin));
-        size_changed           = (view->size != xfce_panel_plugin_get_size(view->plugin));
-        icon_presence_changed  = (view->show_button_icon != cfg->show_button_icon);
-        label_presence_changed = (view->show_button_label != cfg->show_button_label);
-        label_tooltip_changed  = (strcmp(view->label_tooltip_text, cfg->label) != 0);
-        theme_changed          = view->force_update_theme;
+    /* tooltips */
+    new_tooltip_text_hash = g_str_hash(cfg->label);
+    if (new_tooltip_text_hash != tooltip_text_hash) {
+
+#if USE_GTK_TOOLTIP_API
+        gtk_widget_set_tooltip_text(view->button, cfg->label);
+#else
+        gtk_tooltips_set_tip(view->tooltips, view->button, cfg->label, NULL);
+#endif
+
+    }
+    tooltip_text_hash = new_tooltip_text_hash;
+
+}
+/********** Handle user message **********/
+static gboolean
+pview_popup_command_message_received(GtkWidget *widget,
+                                     GdkEventClient *ev,
+                                     PlacesView *view)
+{
+    if (G_LIKELY (ev->data_format == 8 && *(ev->data.b) != '\0'
+                  && !g_ascii_strcasecmp (PLACES_MSG_MENU, ev->data.b))){
+        DBG ("Message received: '%s'", ev->data.b);
+
+        pview_open_menu (view);
+        return TRUE;
     }
 
-    DBG("orientation: %1x, size: %1x, icon_pr: %1x, label_pr: %1x, label_tooltip: %1x",
-        orientation_changed, size_changed, 
-        icon_presence_changed, label_presence_changed, 
-        label_tooltip_changed);
-
-    if(orientation_changed){
-        GtkWidget *button_box = gtk_bin_get_child(GTK_BIN(view->button));
-        
-        view->orientation = xfce_panel_plugin_get_orientation(view->plugin);
-        xfce_hvbox_set_orientation(XFCE_HVBOX(button_box), view->orientation);
-
-        size_changed = TRUE;
-    }
-    
-    if(label_tooltip_changed){
-        g_free(view->label_tooltip_text);
-        view->label_tooltip_text = g_strdup(cfg->label);
-    }
-
-    if(theme_changed)
-        view->force_update_theme = FALSE;
-
-    if(size_changed || icon_presence_changed || label_presence_changed || theme_changed){
-        view->size              = xfce_panel_plugin_get_size(view->plugin);
-        DBG("Panel size: %d", view->size);
-        view->show_button_icon  = cfg->show_button_icon;
-        view->show_button_label = cfg->show_button_label;
-        
-        GdkPixbuf *icon;
-        gint width, height;
-        gint button_overhead_size, box_overhead_size;
-        GtkWidget *button_box = gtk_bin_get_child(GTK_BIN(view->button));
-        
-        button_overhead_size = 2 + 2 *  MAX(view->button->style->xthickness,
-                                            view->button->style->ythickness);
-        box_overhead_size = 0;
-        width = 0;
-        height = 0;
-
-        if(view->show_button_icon){
-            icon = xfce_themed_icon_load_category(2, view->size - button_overhead_size);
-            if(G_LIKELY(icon != NULL)){
-                
-                width  = MAX(gdk_pixbuf_get_width(icon),  view->size - button_overhead_size);
-                height = MAX(gdk_pixbuf_get_height(icon), view->size - button_overhead_size);
-             
-                if(view->button_image == NULL){
-                    view->button_image = g_object_ref(gtk_image_new_from_pixbuf(icon));
-                    gtk_box_pack_start_defaults(GTK_BOX(button_box),
-                                                view->button_image);
-                    gtk_widget_show(view->button_image);
-                }else{
-                    g_assert(GTK_IS_WIDGET(view->button_image));
-                    gtk_image_set_from_pixbuf(GTK_IMAGE(view->button_image), icon);
-                }
-
-                g_object_unref(G_OBJECT(icon));
-            }else{
-                DBG("Could not load icon for button");
-            }
-
-        }else if(view->button_image != NULL){
-            g_assert(GTK_IS_WIDGET(view->button_image));
-            gtk_widget_destroy(view->button_image);
-            g_object_unref(view->button_image);
-            view->button_image = NULL;
-        }
-
-        if(view->show_button_label){
-            GtkRequisition req;
-            
-            if(view->button_label == NULL){
-                view->button_label = g_object_ref(gtk_label_new(cfg->label));
-                gtk_box_pack_end_defaults(GTK_BOX(button_box), 
-                                          view->button_label);
-                gtk_widget_show(view->button_label);
-            }else{
-                g_assert(GTK_IS_WIDGET(view->button_label));
-                if(label_tooltip_changed)
-                    gtk_label_set_text(GTK_LABEL(view->button_label), view->label_tooltip_text);
-            }
-
-            gtk_widget_size_request(view->button_label, &req);
-            if(view->orientation == GTK_ORIENTATION_HORIZONTAL){
-                width += req.width;
-                height = MAX(height, req.height);
-            }else{
-                width = MAX(width, req.width);
-                height += req.height;
-            }
-
-        }else if(view->button_label != NULL){
-            g_assert(GTK_IS_WIDGET(view->button_label));
-            gtk_widget_destroy(view->button_label);
-            g_object_unref(view->button_label);
-            view->button_label = NULL;
-        }
-
-        width += button_overhead_size;
-        height += button_overhead_size;
-
-        if(view->button_image != NULL && view->button_label != NULL){
-
-            box_overhead_size = gtk_box_get_spacing(GTK_BOX(button_box));
-
-            if(view->orientation == GTK_ORIENTATION_HORIZONTAL)
-                width += box_overhead_size;
-            else
-                height += box_overhead_size;
-        }
-
-        if(view->orientation == GTK_ORIENTATION_HORIZONTAL)
-            height = MAX(height, view->size);
-        else
-            width = MAX(width, view->size);
-
-        gtk_widget_show_all(view->button);
-
-        DBG("width=%d, height=%d", width, height);
-        gtk_widget_set_size_request(view->button, width, height);
-    }
-
-    if(label_tooltip_changed)
-        gtk_tooltips_set_tip(view->tooltips, view->button, view->label_tooltip_text, NULL);
+    DBG("Bad message received");
+    return FALSE;
 }
 
-static void
-pview_cfg_dialog_close_cb(GtkDialog *dialog, gint response, PlacesView *view)
+static gboolean
+pview_popup_command_set_selection(PlacesView *view)
 {
-    gtk_widget_destroy(GTK_WIDGET(dialog));
-    xfce_panel_plugin_unblock_menu(view->plugin);
-    places_cfg_view_iface_save(view->cfg_iface);
-}
+    GdkScreen          *gscreen;
+    gchar               selection_name[256];
+    Atom                selection_atom;
+    Window              id;
 
-static GtkWidget*
-pview_make_empty_cfg_dialog(PlacesView *view)
-{
-    GtkWidget *dlg; /* we'll return this */
+    id = GDK_WINDOW_XID (GTK_WIDGET (view->plugin)->window);
 
-    xfce_panel_plugin_block_menu(view->plugin);
+    gscreen = gtk_widget_get_screen (GTK_WIDGET(view->plugin));
+
+    g_snprintf (selection_name, 256,
+                XFCE_PLACES_SELECTION"%d",
+                gdk_screen_get_number (gscreen));
     
-    dlg = xfce_titled_dialog_new_with_buttons(_("Places"),
-              GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(view->plugin))),
-              GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
-              GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
+    selection_atom = XInternAtom (GDK_DISPLAY (), selection_name, FALSE);
 
-    gtk_window_set_position(GTK_WINDOW(dlg), GTK_WIN_POS_CENTER);
-    gtk_window_set_icon_name(GTK_WINDOW(dlg), "xfce4-settings");
+    if (XGetSelectionOwner (GDK_DISPLAY (), selection_atom))
+        return FALSE;
 
-    g_signal_connect(dlg, "response",
-                     G_CALLBACK(pview_cfg_dialog_close_cb), view);
+    XSetSelectionOwner (GDK_DISPLAY (), selection_atom, id, GDK_CURRENT_TIME);
 
-    return dlg;
+    g_signal_connect (GTK_WIDGET(view->plugin),
+                      "client-event",
+                      G_CALLBACK (pview_popup_command_message_received),
+                      view);
+
+    return TRUE;
 }
 
 /********** Initialization & Finalization **********/
 PlacesView*
 places_view_init(XfcePanelPlugin *plugin)
 {
-    GtkWidget *button_box;
     PlacesView *view;                   /* internal use in this file */
     PlacesViewCfgIface *view_cfg_iface; /* given to cfg */
 
@@ -943,59 +897,45 @@ places_view_init(XfcePanelPlugin *plugin)
     view_cfg_iface->update_menu             = pview_update_menu;
     view_cfg_iface->update_button           = pview_button_update;
     view_cfg_iface->reconfigure_model       = pview_reconfigure_model;
-    view_cfg_iface->make_empty_cfg_dialog   = pview_make_empty_cfg_dialog;
-    
     view->view_cfg_iface = view_cfg_iface;
-    view->cfg_iface      = places_cfg_new(view_cfg_iface, 
-                                          xfce_panel_plugin_lookup_rc_file(view->plugin),
-                                          xfce_panel_plugin_save_location(view->plugin, TRUE));
-    view->cfg            = places_cfg_view_iface_get_cfg(view->cfg_iface);
+
+    view->cfg      = places_cfg_new(view->plugin, view_cfg_iface);
 
     pview_reconfigure_model(view);
     
-    view->tooltips = g_object_ref_sink(gtk_tooltips_new());
+#if USE_GTK_TOOLTIP_API
+    DBG("using GtkTooltip API");
+#else
+    DBG("using GtkTooltips API");
+    view->tooltips = exo_gtk_object_ref_sink(GTK_OBJECT(gtk_tooltips_new()));
+#endif
 
     /* init button */
 
     DBG("init GUI");
 
     /* create the button */
-    view->button = g_object_ref(xfce_create_panel_toggle_button());
+    view->button = g_object_ref(places_button_new(view->plugin));
     xfce_panel_plugin_add_action_widget(view->plugin, view->button);
     gtk_container_add(GTK_CONTAINER(view->plugin), view->button);
     gtk_widget_show(view->button);
 
-    /* create the box */
-    button_box = xfce_hvbox_new(xfce_panel_plugin_get_orientation(view->plugin),
-                                FALSE, 4);
-    gtk_container_set_border_width(GTK_CONTAINER(button_box), 0);
-    gtk_container_add(GTK_CONTAINER(view->button), button_box);
-    gtk_widget_show(button_box);
-
     pview_button_update(view);
 
     /* signals for icon theme/screen changes */
     g_signal_connect_swapped(view->button, "style-set",
-                             G_CALLBACK(pview_cb_theme_changed), view);
+                             G_CALLBACK(pview_destroy_menu), view);
     g_signal_connect_swapped(view->button, "screen-changed",
-                             G_CALLBACK(pview_cb_theme_changed), view);
+                             G_CALLBACK(pview_destroy_menu), view);
     
-    /* plugin signals */
-    g_signal_connect_swapped(G_OBJECT(view->plugin), "size-changed",
-                             G_CALLBACK(pview_cb_size_changed), view);
-    g_signal_connect_swapped(G_OBJECT(view->plugin), "orientation-changed",
-                             G_CALLBACK(pview_button_update), view);
-
     /* button signal */
     g_signal_connect_swapped(view->button, "button-press-event",
                              G_CALLBACK(pview_cb_button_pressed), view);
 
-    /* cfg-related signals */
-    g_signal_connect_swapped(G_OBJECT(view->plugin), "configure-plugin",
-                             G_CALLBACK(places_cfg_view_iface_open_dialog), view->cfg_iface);
-    g_signal_connect_swapped(G_OBJECT(view->plugin), "save",
-                             G_CALLBACK(places_cfg_view_iface_save), view->cfg_iface);
-    xfce_panel_plugin_menu_show_configure(view->plugin);
+
+
+    /* set selection for xfce4-popup-places */
+    pview_popup_command_set_selection(view);
 
     DBG("done");
 
@@ -1008,17 +948,14 @@ places_view_finalize(PlacesView *view)
     pview_destroy_menu(view);
     pview_destroy_model(view);
     
-    if(view->button_image != NULL)
-        g_object_unref(view->button_image);
-    if(view->button_label != NULL)
-        g_object_unref(view->button_label);
     if(view->button != NULL)
         g_object_unref(view->button);
-    if(view->label_tooltip_text != NULL)
-        g_free(view->label_tooltip_text);
-    g_object_unref(view->tooltips);
 
-    places_cfg_view_iface_finalize(view->cfg_iface);
+#if !USE_GTK_TOOLTIP_API
+    g_object_unref(view->tooltips);
+#endif
+
+    places_cfg_finalize(view->cfg);
     
     g_free(view->view_cfg_iface);
     g_free(view);
